#!/usr/bin/python3
# -- coding: UTF-8 --

import json
import psutil
import time
import socket
import ctypes
from ctypes import *
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')
import dbus
import dbus.service
import dbus.mainloop.glib
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)
from kysdk_logsec import kylog
from kdk_accessctl import kyaccessctl

BUS_NAME = "com.kylin.kysdk.ListControl"
BUS_PATH = "/com/kylin/kysdk/ListControl"
BUS_IFACE = "com.kylin.kysdk.ListControl"
class ListControl(dbus.service.Object):
    """
    <node>
        <interface name='com.kylin.kysdk.ListControl'>
             <method name='SetWhiteList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetWhiteList' >
                <arg type='as' direction='out' name='white-list'/>
            </method>
            <method name='ClearWhiteList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelWhiteList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <method name='SetBlackList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetBlackList' >
                <arg type='as' direction='out' name='black-list'/>
            </method>
            <method name='ClearBlackList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelBlackList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='SetWhiteListByBatch'>
                <arg type='a(ss)' direction='in' name='whitelist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
            <method name='DelWhiteListByBatch'>
                <arg type='a(ss)' direction='in' name='whitelist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
            <method name='SetBlackListByBatch'>
                <arg type='a(ss)' direction='in' name='blacklist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
            <method name='DelBlackListByBatch'>
                <arg type='a(ss)' direction='in' name='blacklist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        bus = dbus.SystemBus()
        bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
        dbus.service.Object.__init__(self, bus_name, BUS_PATH)

        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")

        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        time = self.get_host_time
        name = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'list-control','lv':'info','message':'{}'".format(
            time, name, ip, message)
        
        try:
            self.log.log_transmit('{' + log_message + '}')
        except Exception:
            print("log message error")


    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def SetWhiteList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)

        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value
 
        if len(pid) > 4 or len(vid) > 4:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList")
            return -1

        white_list = self.GetWhiteList()
        pidvid = pid + vid
        if pidvid in white_list:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList")
            return -1

        if pidvid not in white_list:
            file = open("/etc/mdm/white-list", "a")
            file.write(pidvid + "\n")
            file.close()
        try:
            self.log_transmit("添加白名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        ret = self.so.add_white_list(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="as", sender_keyword='sender')
    def GetWhiteList(self, sender=None) -> list:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetWhiteList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetWhiteList")
            return []

        white_list = []
        try:
            file = open("/etc/mdm/white-list", "r")
            white_list_n = file.readlines()
            for white in white_list_n:
                white_list.append(white.strip("\n"))
        except Exception:
            pass

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetWhiteList")
        return white_list

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def ClearWhiteList(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearWhiteList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)

        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearWhiteList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        try:
            self.log_transmit("清空白名单")
        except Exception:
            print("log message error")

        white_list = []
        try:
            file = open("/etc/mdm/white-list", "r")
            white_list_n = file.readlines()
            for white in white_list_n:
                white_list.append(white.strip("\n"))
        except Exception:
            pass
        file = open("/etc/mdm/white-list", "w")
        file.close()
        for pidvid in white_list:
            self.so.enabledevices(
                ctypes.c_char_p(bytes(pidvid[0:4], encoding="utf8")),
                ctypes.c_char_p(bytes(pidvid[4:8], encoding="utf8")))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearWhiteList")
        return 0

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def DelWhiteList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        pidvid = pid + vid + "\n"
        try:
            file = open("/etc/mdm/white-list", "r")
            white_list_n = file.readlines()
            file.close()
        except Exception:
            white_list_n = []

        if pidvid not in white_list_n:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteList")
            return -1
        white_list_n.remove(pidvid)
        try:
            self.log_transmit("删除白名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        file = open("/etc/mdm/white-list", "w")
        self.so.enabledevices(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))
        for i in white_list_n:
            file.write(i)
        file.close()

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteList")
        return 0

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def SetBlackList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        if len(pid) > 4 or len(vid) > 4:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList")
            return -1

        black_list = self.GetBlackList()
        pidvid = pid + vid
        if pidvid in black_list:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList")
            return -1

        if pidvid not in black_list:
            file = open("/etc/mdm/black-list", "a")
            file.write(pidvid + "\n")
            file.close()

        try:
            self.log_transmit("添加黑名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        ret = self.so.add_black_list(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="as", sender_keyword='sender')
    def GetBlackList(self, sender=None) -> list:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetBlackList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetBlackList")
            return []

        black_list = []
        try:
            file = open("/etc/mdm/black-list", "r")
            black_list_n = file.readlines()
            for black in black_list_n:
                black_list.append(black.strip("\n"))
        except Exception:
            pass

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetBlackList")
        return black_list

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def ClearBlackList(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearBlackList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearBlackList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        try:
            self.log_transmit("清空黑名单")
        except Exception:
            print("log message error")

        black_list = []
        try:
            file = open("/etc/mdm/black-list", "r")
            black_list_n = file.readlines()
            for black in black_list_n:
                black_list.append(black.strip("\n"))
        except Exception:
            pass
        file = open("/etc/mdm/black-list", "w")
        file.close()
        for pidvid in black_list:
            self.so.enabledevices(
                ctypes.c_char_p(bytes(pidvid[0:4], encoding="utf8")),
                ctypes.c_char_p(bytes(pidvid[4:8], encoding="utf8")))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearBlackList")
        return 0

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def DelBlackList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        pidvid = pid + vid + "\n"
        try:
            file = open("/etc/mdm/black-list", "r")
            black_list_n = file.readlines()
            file.close()
        except Exception:
            black_list_n = []

        if pidvid not in black_list_n:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackList")
            return -1
        black_list_n.remove(pidvid)

        try:
            self.log_transmit("删除黑名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        file = open("/etc/mdm/black-list", "w")
        self.so.enabledevices(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))
        for i in black_list_n:
            file.write(i)
        file.close()

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackList")
        return 0

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def SetWhiteListByBatch(self, whitelist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteListByBatch")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        count = ctypes.c_int(count)
        whitelist_items = [a + b for a, b in whitelist]
        whitelist_array = (ctypes.c_char_p * len(whitelist_items))()
        for i, item in enumerate(whitelist_items):
            whitelist_array[i] = ctypes.c_char_p(item.encode('utf-8'))

        print(whitelist_items)
        print(count)
        ret = self.so.kydevice_mdm_add_whitelist_by_batch(whitelist_array, count)

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteListByBatch")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def DelWhiteListByBatch(self, whitelist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteListByBatch")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        count = ctypes.c_int(count)
        whitelist_items = [a + b for a, b in whitelist]
        whitelist_array = (ctypes.c_char_p * len(whitelist_items))()
        for i, item in enumerate(whitelist_items):
            whitelist_array[i] = ctypes.c_char_p(item.encode('utf-8'))

        print(whitelist_items)
        print(count)
        ret = self.so.kydevice_mdm_del_whitelist_by_batch(whitelist_array, count)

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteListByBatch")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def SetBlackListByBatch(self, blacklist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackListByBatch")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        count = ctypes.c_int(count)
        blacklist_items = [a + b for a, b in blacklist]
        blacklist_array = (ctypes.c_char_p * len(blacklist_items))()
        for i, item in enumerate(blacklist_items):
            blacklist_array[i] = ctypes.c_char_p(item.encode('utf-8'))

        print(blacklist_items)
        print(count)
        ret = self.so.kydevice_mdm_add_blacklist_by_batch(blacklist_array, count)

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackListByBatch")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def DelBlackListByBatch(self, blacklist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackListByBatch")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        count = ctypes.c_int(count)
        blacklist_items = [a + b for a, b in blacklist]
        blacklist_array = (ctypes.c_char_p * len(blacklist_items))()
        for i, item in enumerate(blacklist_items):
            blacklist_array[i] = ctypes.c_char_p(item.encode('utf-8'))

        print(blacklist_items)
        print(count)
        ret = self.so.kydevice_mdm_del_blacklist_by_batch(blacklist_array, count)

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackListByBatch")
        return ret

if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = dbus.SystemBus()
    bus.request_name(BUS_NAME)
    ListControl()
    loop.run()
