#!/usr/bin/python3
# -- coding: UTF-8 --

import json
import psutil
import time
import socket
import ctypes
from ctypes import *
from pydbus import SystemBus
import dbus
import dbus.service
import dbus.mainloop.glib
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')
from kysdk_logsec import kylog
from kdk_accessctl import kyaccessctl

MDM_DEV_MODULE_CD = 2
MDM_DEV_STATE_DISBALE = 0
MDM_DEV_STATE_READONLY = 1
MDM_DEV_STATE_ENABLE = 2

BUS_NAME = "com.kylin.kysdk.CDControl"
BUS_PATH = "/com/kylin/kysdk/CDControl"
BUS_IFACE = "com.kylin.kysdk.CDControl"
class CDControl(dbus.service.Object):
    """
    <node>
        <interface name='com.kylin.kysdk.CDControl'>

            <method name='SetStatus' >
                <arg type='i' direction='in' name='status'/>
                <arg type='i' direction='out' name='ret'/>
            </method>

            <method name='GetStatus' >
                <arg type='i' direction='out' name='status'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        bus = dbus.SystemBus()
        bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
        dbus.service.Object.__init__(self, bus_name, BUS_PATH)

        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")
        self.func_dict = {
            0: "DisableCD",
            1: "ReadonlyCD",
            2: "EnableCD"
        }

        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        time = self.get_host_time
        name = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'cd-monitor','lv':'info','message':'{}'".format(
            time, name, ip, message)

        try:
            self.log.log_transmit('{' + log_message + '}')
        except Exception:
            print("log message error")

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="i", out_signature="i", sender_keyword='sender')
    def SetStatus(self, status: int, sender=None) -> int:
        """
        status
            0   禁用
            1   只读
            2   不管控
        """
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetStatus")
        
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetStatus")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        
        if status in self.func_dict.keys():
            # 通过status调用对应函数
            ret = getattr(self, self.func_dict.get(status))()
        else:
            ret = -1

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetStatus")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def GetStatus(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetStatus")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetStatus")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        try:
            file = open("/etc/mdm/cd", "r")
            value = file.read(1)
            file.close()
        except Exception:
            value = '2'
        if len(value) == 0 or value < '0' or value > '2':
            value = '2'

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetStatus")
        return int(value)

    def DisableCD(self) -> int:
        try:
            self.log_transmit("开启光驱设备禁用管控")
        except Exception:
            print("log message error")
        
        if self.GetStatus() == 0:
            return 0
        file = open("/etc/mdm/cd", "w")
        file.write("0")
        file.close()

        #禁用USB光驱
        self.so.disable_dev_module(c_int(MDM_DEV_MODULE_CD))
        #禁用内置光驱
        self.so.set_ata_cd_dev_status(c_int(MDM_DEV_STATE_DISBALE))
        return 0

    def EnableCD(self) -> int:
        try:
            self.log_transmit("关闭光驱设备管控")
        except Exception:
            print("log message error")

        if self.GetStatus() == 2:
            return 0

        file = open("/etc/mdm/cd", "w")
        file.write("2")
        file.close()

        try:
            blacklist_f = open("/etc/mdm/black-list", "r")
            blacklist = blacklist_f.readlines()
            blacklist_f.close()
        except Exception:
            blacklist = []

        try:
            disablelist_f = open("/etc/mdm/disable-cd", "r")
            disablelist = disablelist_f.readlines()
            disablelist_f.close()
        except Exception:
            disablelist = []

        # 启用USB光驱
        for cd in disablelist:
            if cd not in blacklist:
                self.so.enable_dev_by_pidvid(
                    ctypes.c_char_p(bytes(cd[0:4], encoding="utf8")),
                    ctypes.c_char_p(bytes(cd[4:8], encoding="utf8")))

        # 启用内置光驱
        self.so.set_ata_cd_dev_enable()
        return 0

    def ReadonlyCD(self) -> int:
        try:
            self.log_transmit("开启光驱设备只读管控")
        except Exception:
            print("log message error")
            
        if self.GetStatus() == 1:
            return 0

        file = open("/etc/mdm/cd", "w")
        file.write("1")
        file.close()

        try:
            blacklist_f = open("/etc/mdm/black-list", "r")
            blacklist = blacklist_f.readlines()
            blacklist_f.close()
        except Exception:
            blacklist = []

        try:
            disablelist_f = open("/etc/mdm/disable-cd", "r")
            disablelist = disablelist_f.readlines()
            disablelist_f.close()
        except Exception:
            disablelist = []

        # 已禁用设备先设置启用
        for cd in disablelist:
            if cd not in blacklist:
                self.so.enable_dev_by_pidvid(
                    ctypes.c_char_p(bytes(cd[0:4], encoding="utf8")),
                    ctypes.c_char_p(bytes(cd[4:8], encoding="utf8")))

        # 设置USB光驱只读
        self.so.readonly_dev_module(c_int(MDM_DEV_MODULE_CD))
        # 设置内置光驱只读
        self.so.set_ata_cd_dev_readonly()
        return 0


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = dbus.SystemBus()
    bus.request_name(BUS_NAME)
    CDControl()
    loop.run()
